# Additive Increase, Multiplicative Decrease (AIMD)

Un posibil algoritm de evitare a congestiei este AIMD. **Algoritmul crește fereastra de congestie cu un MSS per Round Trip Time (RTT)**.

Există două implementări posibile pentru partea de creștere a ferestrei: actualizarea ferestrei la fiecare ACK, sau o dată per RTT.

Algoritmul pentru actualizare la fiecare ACK este mai simplu de implementat pentru că nu necesită menținerea unei variabile suplimentare care să detecteze când a trecut un RTT. Să presupunem că fereastra de congestie este menținută în octeti. În acest caz, la primirea unui ACK,
fereastra de congestie crește astfel.

```
onAck(bytes_acked - numărul de octeți confirmați de receptor)
    if (CWND > ssthresh) //additive increase
        CWND = CWND + bytes_acked * MSS / CWND
    else //slow start
        CWND = CWND + MSS
```
De-a lungul unui round-trip time, suma bytes_acked va fi egală cu CWND; astfel, creșterea totală va fi MSS per RTT (additive increase).

Variabila `ssthresh` face trecerea dintre Slow Start și Additive Increase. Atunci când conexiunea începe, Slow Start va avea valoarea MAXINT. După fiecare pierdere, `ssthresh` este actualizată pentru a memora o valoare "sigură" a ferestrei de congestie; atunci când valoarea CWND este sub ssthresh, creșterea este exponențială. Aceasta asigura că fereastra crește rapid la începutul conexiunii pentru a utiliza rapid capacitatea rețelei.

---
**Atenție!**

Dacă toate variabilele folosite sunt întregi, există riscul ca atunci când CWND are valori mari, creșterea bytes_acked * MSS să fie mai mică decât CWND și astfel fereastră să nu mai crească! Pentru a evita astfel de erori de rotunjire, se vor folosi valori floating point pentru CWND.

---

**Atunci când un packet este pierdut, fereastra de congestie este redusă la jumătate (multiplicative decrease)**. De notat este că se va reduce fereastră o dată per loss event - chiar dacă se pierd mai multe pachete într-un RTT, fereastra este redusă o singură dată.

```
onLoss(detectată cu ack-uri duplicate)
    CWND = CWND / 2
    ssthresh = CWND
```

Atunci când nu primim confirmarea pentru un packet și expiră timer-ul, se va execută codul de mai jos care reduce CWND și mai agresiv:

```
onTimeout():
    ssthresh = CWND/2
    CWND = 1 MSS
```

Comportamentul dat de AIMD de evitare a congestiei este surprins în figura de mai jos
și se mai numește "dinți de fierăstrău". 

![](images/sawtooth.png)






